/*
	*
	* Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
	*

	*
	* This file is a part of DesQ project ( https://gitlab.com/desq/ )
	* Qt5 example of analog clock, and Lenovo K8 Note's clock widget
	* are the inspiration for this. Some parts of the code are taken
	* from Qt5's example
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* ( at your option ) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include "Global.hpp"
#include "DesQDock.hpp"
#include "DesQDockPager.hpp"
#include "DesQDockTasks.hpp"

#include <desqui/DesQLayout.hpp>
#include <desqwl/DesQRegistry.hpp>

QWidget *separator( Qt::Orientation direction ) {

	QWidget *sep = new QWidget();
	sep->setStyleSheet( "background-color: gray;" );
	switch( direction ) {
		case Qt::Horizontal: {

			sep->setFixedHeight( 1 );
			sep->setMinimumWidth( 1 );
			sep->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed ) );
			break;
		}

		case Qt::Vertical: {

			sep->setMinimumHeight( 1 );
			sep->setFixedWidth( 1 );
			sep->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::MinimumExpanding ) );
			break;
		}
	}

	return sep;
};

DesQDock::DesQDock( DesQWaylandRegistry *wlRegistry ) : QWidget() {

	/* No frame, and stay at bottom */
	Qt::WindowFlags wFlags = Qt::FramelessWindowHint | Qt::WindowStaysOnBottomHint | Qt::BypassWindowManagerHint;

	setAttribute( Qt::WA_TranslucentBackground );

	/* Set the windowFlags */
	setWindowFlags( wFlags );

	setWindowTitle( tr( "DesQ Dock" ) );

	desqBtn = new QToolButton();
	desqBtn->setIcon( QIcon::fromTheme( "desq" ) );
	desqBtn->setAutoRaise( true );
	desqBtn->setFixedSize( QSize( 32, 32 ) );
	desqBtn->setIconSize( QSize( 28, 28 ) );
	desqBtn->setStyleSheet(
		"QToolButton {"
		"	border: none;"
		"	border-radius: 16px;"
		"}"
		"QToolButton:hover{"
		"	border: 1px solid palette(Highlight);"
		"}"
		"QToolButton:pressed {"
		"	background-color: palette(Highlight);"
		"	padding-left: 1px;"
		"	padding-top: 1px;"
		"}"
	);

	connect( desqBtn, &QToolButton::clicked, this, &DesQDock::newWindow );

	pager = new DesQDockPager( this );
	tasks = new DesQDockTasks( wlRegistry->windowManager(), this );

	dateTimeLbl = new QLabel();
	dateTimeLbl->setAlignment( Qt::AlignCenter );

	clockTimer = new QBasicTimer();
	clockTimer->start( 500, this );

	infoBtn = new QToolButton();
	infoBtn->setIcon( QIcon::fromTheme( "info" ) );
	infoBtn->setAutoRaise( true );
	infoBtn->setFixedSize( QSize( 32, 32 ) );
	infoBtn->setIconSize( QSize( 28, 28 ) );
	infoBtn->setStyleSheet(
		"QToolButton {"
		"	border: none;"
		"	border-radius: 16px;"
		"}"
		"QToolButton:hover{"
		"	border: 1px solid palette(Highlight);"
		"}"
		"QToolButton:pressed {"
		"	background-color: palette(Highlight);"
		"	padding-left: 1px;"
		"	padding-top: 1px;"
		"}"
	);

	lyt = new QHBoxLayout( this );
	lyt->setContentsMargins( QMargins( 2, 2, 2, 2 ) );
	lyt->setSpacing( 5 );
	lyt->addWidget( desqBtn );
	lyt->addWidget( separator( Qt::Vertical ) );
	lyt->addWidget( pager );
	lyt->addWidget( separator( Qt::Vertical ) );
	lyt->addWidget( tasks );
	lyt->addWidget( separator( Qt::Vertical ) );
	lyt->addStretch();
	lyt->addWidget( dateTimeLbl );
	lyt->addWidget( infoBtn );

	setMinimumWidth( 200 );
	setFixedHeight( 36 );

	setLayout( lyt );
};

void DesQDock::show() {

	QWidget::show();
	QTimer::singleShot( 5000, this, &DesQDock::hide );
};

void DesQDock::hide() {

	if ( underMouse() ) {
		QTimer::singleShot( 5000, this, &DesQDock::hide );
		return;
	}

	else
		QWidget::hide();
};

void DesQDock::paintEvent( QPaintEvent *pEvent ) {

	QPainter painter( this );
	painter.setRenderHint( QPainter::Antialiasing );

	painter.save();
	painter.setBrush( QColor( 0, 0, 0, 210 ) );
	painter.setPen( Qt::NoPen );
	painter.drawRoundedRect( QRect( 0, 0, width(), height() ), height() / 2, height() / 2 );
	painter.restore();

	painter.end();

	QWidget::paintEvent( pEvent );
};

void DesQDock::timerEvent( QTimerEvent *tEvent ) {

	if ( tEvent->timerId() == clockTimer->timerId() ) {
		dateTimeLbl->setText( "<center>" + QDateTime::currentDateTime().toString( "hh:mm:ss AP<br>ddd, MMM dd, yyyy" ) + "</center>" );
	};

	QWidget::timerEvent( tEvent );
};
